@@ -1,13 +1,13 @@
-#' Wrap Arbitrary Graphics for Alignment
+#' Wrap Arbitrary Graphics to ggplot
 #'
-#' The `wrap()` function allows non-ggplot2 elements to be converted into a
-#' compliant representation for use with [align_plots()]. This is useful for
-#' adding any graphics that can be converted into a [grob][grid::grob] with the
-#' [patch()] method.
+#' The `ggwrap()` function allows non-ggplot2 elements to be converted into a
+#' compliant representation for use with [`align_plots()`]. This is useful for
+#' adding any graphics that can be converted into a [`grob`][grid::grob] with
+#' the [`patch()`] method.
 #'
-#' @param plot Any graphic that can be converted into a [grob][grid::grob] using
-#' [patch()].
-#' @param ... Additional arguments passed to the [patch()] method.
+#' @param plot Any graphic that can be converted into a [`grob`][grid::grob]
+#' using [`patch()`].
+#' @param ... Additional arguments passed to the [`patch()`] method.
 #' @param align A string specifying the area to place the plot: `"full"` for the
 #' full area, `"plot"` for the full plotting area (including the axis label), or
 #' `"panel"` for only the actual area where data is drawn.
@@ -16,14 +16,14 @@
 #' @param on_top A single boolean value indicates whether the graphic plot
 #' should be put frontmost. Note: the graphic plot will always put above the
 #' background.
-#' @param vp A [viewport][grid::viewport] object, you can use this to define the
-#' plot area.
+#' @param vp A [`viewport`][grid::viewport] object, you can use this to define
+#' the plot area.
 #' @return A `wrapped_plot` object that can be directly placed into
-#' [align_plots()].
+#' [`align_plots()`].
 #' @inherit patch seealso
 #' @examples
 #' library(grid)
-#' wrap(rectGrob(gp = gpar(fill = "goldenrod")), align = "full") +
+#' ggwrap(rectGrob(gp = gpar(fill = "goldenrod")), align = "full") +
 #'     inset(rectGrob(gp = gpar(fill = "steelblue")), align = "panel") +
 #'     inset(textGrob("Here are some text", gp = gpar(color = "black")),
 #'         align = "panel"
@@ -31,17 +31,17 @@
 #' p1 <- ggplot(mtcars) +
 #'     geom_point(aes(mpg, disp)) +
 #'     ggtitle("Plot 1")
-#' align_plots(p1, wrap(
+#' align_plots(p1, ggwrap(
 #'     ~ plot(mtcars$mpg, mtcars$disp),
 #'     mar = c(0, 2, 0, 0), bg = NA
 #' ))
 #'
-#' @importFrom ggplot2 theme element_blank
+#' @importFrom ggplot2 theme element_blank ggplot
 #' @importFrom grid is.grob
 #' @export
-wrap <- function(plot, ..., align = "panel", on_top = TRUE,
-                 clip = TRUE, vp = NULL) {
-    patch <- ggplot2::ggplot() +
+ggwrap <- function(plot, ..., align = "panel", on_top = TRUE,
+                   clip = TRUE, vp = NULL) {
+    patch <- ggplot() +
         theme(
             plot.background = element_blank(),
             panel.background = element_blank()
@@ -93,10 +93,8 @@ alignpatch.wrapped_plot <- function(x) {
         "PatchWrapped", Parent,
         wrapped_grobs_under = attr(x, "wrapped_grobs_under"),
         wrapped_grobs_above = attr(x, "wrapped_grobs_above"),
-        patch_gtable = function(self, guides, plot = Parent$plot) {
-            ans <- ggproto_parent(Parent, self)$patch_gtable(
-                guides = guides, plot = plot
-            )
+        patch_gtable = function(self, plot = Parent$plot) {
+            ans <- ggproto_parent(Parent, self)$patch_gtable(plot = plot)
             ans <- add_wrapped_grobs(ans, self$wrapped_grobs_under, FALSE)
             add_wrapped_grobs(ans, self$wrapped_grobs_above, TRUE)
         }
@@ -105,7 +103,7 @@ alignpatch.wrapped_plot <- function(x) {
 
 # For wrapped plot -------------------
 #' @export
-alignpatch.grob <- function(x) alignpatch(wrap(x))
+alignpatch.grob <- function(x) alignpatch(ggwrap(x))
 
 #' @export
 alignpatch.gList <- alignpatch.grob
@@ -123,7 +121,7 @@ alignpatch.recordedplot <- alignpatch.grob
 alignpatch.trellis <- alignpatch.grob
 
 #' @export
-alignpatch.Heatmap <- function(x) alignpatch(wrap(x, align = "full"))
+alignpatch.Heatmap <- function(x) alignpatch(ggwrap(x, align = "full"))
 
 #' @export
 alignpatch.HeatmapList <- alignpatch.Heatmap
@@ -132,7 +130,7 @@ alignpatch.HeatmapList <- alignpatch.Heatmap
 alignpatch.HeatmapAnnotation <- alignpatch.Heatmap
 
 #' @export
-alignpatch.pheatmap <- function(x) alignpatch(wrap(x, align = "full"))
+alignpatch.pheatmap <- function(x) alignpatch(ggwrap(x, align = "full"))
 
 ################################################## 3
 add_wrapped_grobs <- function(gt, grobs, on_top) {
